@@ -189,24 +189,6 @@ static int send_reply(fuse_req_t req, int error, const void *arg,
     return send_reply_iov(req, error, iov, count);
 }
 
-int fuse_reply_iov(fuse_req_t req, const struct iovec *iov, int count)
-{
-    int res;
-    struct iovec *padded_iov;
-
-    padded_iov = malloc((count + 1) * sizeof(struct iovec));
-    if (padded_iov == NULL)
-        return fuse_reply_err(req, -ENOMEM);
-
-    memcpy(padded_iov + 1, iov, count * sizeof(struct iovec));
-    count++;
-
-    res = send_reply_iov(req, 0, padded_iov, count);
-    free(padded_iov);
-
-    return res;
-}
-
 size_t fuse_dirent_size(size_t namelen)
 {
     return FUSE_DIRENT_ALIGN(FUSE_NAME_OFFSET + namelen);
@@ -1244,13 +1226,8 @@ static void fuse_ll_destroy(void *data)
     free(f);
 }
 
-/*
- * always call fuse_lowlevel_new_common() internally, to work around a
- * misfeature in the FreeBSD runtime linker, which links the old
- * version of a symbol to internal references.
- */
-struct fuse_session *fuse_lowlevel_new_common(struct fuse_args *args,
-                                       const struct fuse_lowlevel_ops *op,
+struct fuse_session *fuse_lowlevel_new(struct fuse_args *args,
+				       const struct fuse_lowlevel_ops *op,
                                        size_t op_size, void *userdata)
 {
     struct fuse_ll *f;
@@ -1297,24 +1274,3 @@ struct fuse_session *fuse_lowlevel_new_common(struct fuse_args *args,
     return NULL;
 }
 
-
-struct fuse_session *fuse_lowlevel_new(struct fuse_args *args,
-                                       const struct fuse_lowlevel_ops *op,
-                                       size_t op_size, void *userdata)
-{
-    return fuse_lowlevel_new_common(args, op, op_size, userdata);
-}
-
-
-struct fuse_ll_compat_conf {
-    unsigned max_read;
-    int set_max_read;
-};
-
-static const struct fuse_opt fuse_ll_opts_compat[] = {
-    { "max_read=", offsetof(struct fuse_ll_compat_conf, set_max_read), 1 },
-    { "max_read=%u", offsetof(struct fuse_ll_compat_conf, max_read), 0 },
-    FUSE_OPT_KEY("max_read=", FUSE_OPT_KEY_KEEP),
-    FUSE_OPT_END
-};
-
