@@ -156,7 +156,7 @@ ntfs_inode *ntfs_inode_open(ntfs_volume *vol, const MFT_REF mref)
 	ntfs_attr_search_ctx *ctx;
 	STANDARD_INFORMATION *std_info;
 
-	ntfs_log_enter("Entering for inode %lld\n", MREF(mref));
+	ntfs_log_enter("Entering for inode %lld\n", (long long)MREF(mref));
 	if (!vol) {
 		errno = EINVAL;
 		goto out;
@@ -177,8 +177,7 @@ ntfs_inode *ntfs_inode_open(ntfs_volume *vol, const MFT_REF mref)
 	/* Receive some basic information about inode. */
 	if (ntfs_attr_lookup(AT_STANDARD_INFORMATION, AT_UNNAMED,
 				0, CASE_SENSITIVE, 0, NULL, 0, ctx)) {
-		ntfs_log_trace("Failed to receive STANDARD_INFORMATION "
-				"attribute.\n");
+		ntfs_log_perror("No STANDARD_INFORMATION in base record\n");
 		goto put_err_out;
 	}
 	std_info = (STANDARD_INFORMATION *)((u8 *)ctx->attr +
@@ -202,6 +201,7 @@ ntfs_inode *ntfs_inode_open(ntfs_volume *vol, const MFT_REF mref)
 		goto put_err_out;
 	if (l > 0x40000) {
 		errno = EIO;
+		ntfs_log_perror("Too large attrlist (%lld)\n", (long long)l);
 		goto put_err_out;
 	}
 	ni->attr_list_size = l;
@@ -213,6 +213,8 @@ ntfs_inode *ntfs_inode_open(ntfs_volume *vol, const MFT_REF mref)
 		goto put_err_out;
 	if (l != ni->attr_list_size) {
 		errno = EIO;
+		ntfs_log_perror("Unexpected attrlist size (%lld <> %u)\n",
+				(long long)l, ni->attr_list_size);
 		goto put_err_out;
 	}
 get_size:
@@ -512,20 +514,17 @@ static int ntfs_inode_sync_standard_information(ntfs_inode *ni)
 {
 	ntfs_attr_search_ctx *ctx;
 	STANDARD_INFORMATION *std_info;
-	int err;
 
-	ntfs_log_trace("Entering for inode 0x%llx.\n", (long long) ni->mft_no);
+	ntfs_log_trace("Entering for inode %lld\n", (long long)ni->mft_no);
 
 	ctx = ntfs_attr_get_search_ctx(ni, NULL);
 	if (!ctx)
 		return -1;
 	if (ntfs_attr_lookup(AT_STANDARD_INFORMATION, AT_UNNAMED,
-				0, CASE_SENSITIVE, 0, NULL, 0, ctx)) {
-		err = errno;
-		ntfs_log_trace("Failed to receive STANDARD_INFORMATION "
-				"attribute.\n");
+			     0, CASE_SENSITIVE, 0, NULL, 0, ctx)) {
+		ntfs_log_perror("Failed to sync standard info (inode %lld)",
+				(long long)ni->mft_no);
 		ntfs_attr_put_search_ctx(ctx);
-		errno = err;
 		return -1;
 	}
 	std_info = (STANDARD_INFORMATION *)((u8 *)ctx->attr +
@@ -686,8 +685,6 @@ int ntfs_inode_sync(ntfs_inode *ni)
 			if (err != EIO)
 				err = EBUSY;
 		}
-		ntfs_log_perror("Failed to sync standard info (inode %lld)",
-				(long long)ni->mft_no);
 	}
 
 	/* Update FILE_NAME's in the index. */
@@ -986,8 +983,8 @@ err_out:
 }
 
 /**
- * ntfs_inode_free_space - free space in the MFT record of inode
- * @ni:		ntfs inode in which MFT record free space
+ * ntfs_inode_free_space - free space in the MFT record of an inode
+ * @ni:		ntfs inode in which MFT record needs more free space
  * @size:	amount of space needed to free
  *
  * Return 0 on success or -1 on error with errno set to the error code.
@@ -995,7 +992,7 @@ err_out:
 int ntfs_inode_free_space(ntfs_inode *ni, int size)
 {
 	ntfs_attr_search_ctx *ctx;
-	int freed, err;
+	int freed;
 
 	if (!ni || size < 0) {
 		errno = EINVAL;
@@ -1015,97 +1012,60 @@ int ntfs_inode_free_space(ntfs_inode *ni, int size)
 	ctx = ntfs_attr_get_search_ctx(ni, NULL);
 	if (!ctx)
 		return -1;
-
 	/*
-	 * Chkdsk complain if $STANDARD_INFORMATION is not in the base MFT
-	 * record. FIXME: I'm not sure in this, need to recheck. For now simply
-	 * do not move $STANDARD_INFORMATION at all.
-	 *
-	 * Also we can't move $ATTRIBUTE_LIST from base MFT_RECORD, so position
-	 * search context on first attribute after $STANDARD_INFORMATION and
-	 * $ATTRIBUTE_LIST.
-	 *
-	 * Why we reposition instead of simply skip this attributes during
-	 * enumeration? Because in case we have got only in-memory attribute
-	 * list ntfs_attr_lookup will fail when it will try to find
-	 * $ATTRIBUTE_LIST.
+	 * $STANDARD_INFORMATION and $ATTRIBUTE_LIST must stay in the base MFT
+	 * record, so position search context on the first attribute after them.
 	 */
-	if (ntfs_attr_lookup(AT_FILE_NAME, NULL, 0, CASE_SENSITIVE, 0, NULL,
-				0, ctx)) {
-		if (errno != ENOENT) {
-			err = errno;
-			ntfs_log_perror("%s: attr lookup failed #2", __FUNCTION__);
-			goto put_err_out;
-		}
-		if (ctx->attr->type == AT_END) {
-			err = ENOSPC;
-			goto put_err_out;
-		}
-	}
+	if (ntfs_attr_position(AT_FILE_NAME, ctx))
+		goto put_err_out;
 
 	while (1) {
 		int record_size;
-
 		/*
 		 * Check whether attribute is from different MFT record. If so,
 		 * find next, because we don't need such.
 		 */
 		while (ctx->ntfs_ino->mft_no != ni->mft_no) {
 retry:			
-			if (ntfs_attr_lookup(AT_UNUSED, NULL, 0, CASE_SENSITIVE,
-						0, NULL, 0, ctx)) {
-				err = errno;
-				if (errno != ENOENT) {
-					ntfs_log_perror("Attr lookup failed #2");
-				} else
-					err = ENOSPC;
+			if (ntfs_attr_position(AT_UNUSED, ctx))
 				goto put_err_out;
-			}
 		}
 
 		if (ntfs_inode_base(ctx->ntfs_ino)->mft_no == FILE_MFT && 
 		    ctx->attr->type == AT_DATA)
 			goto retry;
 
+		if (ctx->attr->type == AT_INDEX_ROOT)
+			goto retry;
+
 		record_size = le32_to_cpu(ctx->attr->length);
 
-		/* Move away attribute. */
 		if (ntfs_attr_record_move_away(ctx, 0)) {
-			err = errno;
 			ntfs_log_perror("Failed to move out attribute #2");
 			break;
 		}
 		freed += record_size;
 
-		/* Check whether we done. */
+		/* Check whether we are done. */
 		if (size <= freed) {
 			ntfs_attr_put_search_ctx(ctx);
 			return 0;
 		}
-
 		/*
-		 * Reposition to first attribute after $STANDARD_INFORMATION and
-		 * $ATTRIBUTE_LIST (see comments upwards).
+		 * Reposition to first attribute after $STANDARD_INFORMATION 
+		 * and $ATTRIBUTE_LIST instead of simply skipping this attribute 
+		 * because in the case when we have got only in-memory attribute 
+		 * list then ntfs_attr_lookup will fail when it tries to find 
+		 * $ATTRIBUTE_LIST.
 		 */
 		ntfs_attr_reinit_search_ctx(ctx);
-		if (ntfs_attr_lookup(AT_FILE_NAME, NULL, 0, CASE_SENSITIVE, 0,
-				NULL, 0, ctx)) {
-			if (errno != ENOENT) {
-				err = errno;
-				ntfs_log_perror("Attr lookup #2 failed");
-				break;
-			}
-			if (ctx->attr->type == AT_END) {
-				err = ENOSPC;
-				break;
-			}
-		}
+		if (ntfs_attr_position(AT_FILE_NAME, ctx))
+			break;
 	}
 put_err_out:
 	ntfs_attr_put_search_ctx(ctx);
-	if (err == ENOSPC)
-		ntfs_log_trace("No attributes left that can be moved out.\n");
-	errno = err;
+	if (errno == ENOSPC)
+		ntfs_log_trace("No attributes left that could be moved out.\n");
 	return -1;
 }
 
